"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;
var _clientLambda = require("@aws-sdk/client-lambda");
var _nodeHttpHandler = require("@smithy/node-http-handler");
var _bluebird = _interopRequireDefault(require("bluebird"));
var _lodash = require("lodash");
var _log = require("../utils/log");
/* eslint import/no-extraneous-dependencies: ["error", {"devDependencies": true}] */

class Connector {
  constructor({
    debug,
    pipelineId,
    timeout = Number(process.env.LAMBDA_TIMEOUT) || Number(process.env.TIMEOUT) || 1000,
    additionalClientOpts = {},
    ...opt
  }) {
    this.debug = msg => debug('%j', msg);
    this.client = Connector.getClient(pipelineId, debug, timeout, additionalClientOpts);
    this.opt = opt;
  }
  static clients = {};
  static getClient(pipelineId, debug, timeout, additionalClientOpts) {
    const addlRequestHandlerOpts = (0, _lodash.pick)(additionalClientOpts, ['requestHandler']);
    const addlClientOpts = (0, _lodash.omit)(additionalClientOpts, ['requestHandler']);
    if (!this.clients[pipelineId]) {
      this.clients[pipelineId] = new _clientLambda.LambdaClient({
        requestHandler: new _nodeHttpHandler.NodeHttpHandler({
          requestTimeout: timeout,
          connectionTimeout: timeout,
          ...addlRequestHandlerOpts
        }),
        logger: (0, _log.defaultDebugLogger)(debug),
        ...addlClientOpts
      });
    }
    return this.clients[pipelineId];
  }
  invoke(params, ctx) {
    return this._sendCommand(new _clientLambda.InvokeCommand(params), ctx);
  }
  _sendCommand(command, ctx) {
    this.opt.metrics?.capture(this.client, command, 'lambda', this.opt, ctx);
    return _bluebird.default.resolve(this.client.send(command)).tap(this.debug).tapCatch(this.debug);
  }
}
var _default = exports.default = Connector;