"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;
var _stream = require("stream");
var _clientS = require("@aws-sdk/client-s3");
var _nodeHttpHandler = require("@smithy/node-http-handler");
var _bluebird = _interopRequireDefault(require("bluebird"));
var _lodash = require("lodash");
var _log = require("../utils/log");
/* eslint import/no-extraneous-dependencies: ["error", {"devDependencies": true}] */
// import { S3, config } from 'aws-sdk';

class Connector {
  constructor({
    debug,
    pipelineId,
    bucketName = process.env.BUCKET_NAME,
    timeout = Number(process.env.S3_TIMEOUT) || Number(process.env.TIMEOUT) || 1000,
    credentials,
    additionalClientOpts = {},
    ...opt
  }) {
    this.debug = msg => debug('%j', msg);
    this.bucketName = bucketName || 'undefined';
    this.client = Connector.getClient(pipelineId, debug, timeout, credentials, additionalClientOpts);
    this.opt = opt;
  }
  static clients = {};
  static getClient(pipelineId, debug, timeout, credentials, additionalClientOpts) {
    const addlRequestHandlerOpts = (0, _lodash.pick)(additionalClientOpts, ['requestHandler']);
    const addlClientOpts = (0, _lodash.omit)(additionalClientOpts, ['requestHandler']);
    if (!this.clients[pipelineId]) {
      this.clients[pipelineId] = new _clientS.S3Client({
        credentials,
        requestHandler: new _nodeHttpHandler.NodeHttpHandler({
          requestTimeout: timeout,
          connectionTimeout: timeout,
          ...addlRequestHandlerOpts
        }),
        logger: (0, _log.defaultDebugLogger)(debug),
        ...addlClientOpts
      });
    }
    return this.clients[pipelineId];
  }
  putObject(inputParams, ctx) {
    const params = {
      Bucket: this.bucketName,
      ...inputParams
    };
    return this._sendCommand(new _clientS.PutObjectCommand(params), ctx);
  }
  deleteObject(inputParams, ctx) {
    const params = {
      Bucket: this.bucketName,
      ...inputParams
    };
    return this._sendCommand(new _clientS.DeleteObjectCommand(params), ctx);
  }
  getObject(inputParams, ctx) {
    const params = {
      Bucket: this.bucketName,
      ...inputParams
    };
    return this._sendCommand(new _clientS.GetObjectCommand(params), ctx).then(async response => ({
      ...response,
      Body: await response.Body.transformToString()
    }));
  }
  getObjectStream(inputParams, ctx) {
    const params = {
      Bucket: this.bucketName,
      ...inputParams
    };
    return this._sendCommand(new _clientS.GetObjectCommand(params), ctx).then(response => _stream.Readable.from(response.Body));
  }
  listObjects(inputParams, ctx) {
    const params = {
      Bucket: this.bucketName,
      ...inputParams
    };
    return this._sendCommand(new _clientS.ListObjectsV2Command(params), ctx);
  }
  copyObject(inputParams, ctx) {
    const params = {
      Bucket: this.bucketName,
      ...inputParams
    };
    return this._sendCommand(new _clientS.CopyObjectCommand(params), ctx);
  }
  _sendCommand(command, ctx) {
    this.opt.metrics?.capture(this.client, command, 's3', this.opt, ctx);
    return _bluebird.default.resolve(this.client.send(command)).tap(this.debug).tapCatch(this.debug);
  }
}
var _default = exports.default = Connector;