"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.toSqsRecords = exports.toSqsEventRecords = exports.fromSqsEvent = exports.fromSqs = exports.UNKNOWN_SQS_EVENT_TYPE = void 0;
var _highland = _interopRequireDefault(require("highland"));
var _utils = require("../utils");
var _filters = require("../filters");
// this from function is intended for use with intra-service messages
// as opposed to consuming inter-servic events

const fromSqs = event => (0, _highland.default)(event.Records).map(record => (
// create a unit-of-work for each message
// so we can correlate related work for error handling
{
  record
})).tap(uow => (0, _utils.options)().metrics?.adornSqsMetrics(uow, event));
exports.fromSqs = fromSqs;
const fromSqsEvent = event => (0, _highland.default)(event.Records)
// create a unit-of-work for each event
// so we can correlate related work for error handling
.map((0, _utils.faulty)(record => ({
  record,
  event: {
    id: record.messageId,
    ...JSON.parse(record.body, _utils.decompress)
  }
}))).tap(uow => (0, _utils.options)().metrics?.adornSqsMetrics(uow, event)).filter(_filters.outSkip).through((0, _utils.claimcheck)());

// test helper
// https://docs.aws.amazon.com/lambda/latest/dg/with-sqs.html
exports.fromSqsEvent = fromSqsEvent;
const toSqsRecords = messages => ({
  Records: messages.map((m, i) => ({
    eventSource: 'aws:sqs',
    messageId: `00000000-0000-0000-0000-00000000000${i}`,
    // receiptHandle: 'AQEBwJnKyrHigUMZj6rYigCgxlaS3SLy0a...',
    body: m.body,
    attributes: {
      // ApproximateReceiveCount: '1',
      SentTimestamp: m.timestamp || /* istanbul ignore next */'1545082649183'
      // SenderId: 'AIDAIENQZJOLO23YVJ4VO',
      // ApproximateFirstReceiveTimestamp: '1545082649185'

      // SequenceNumber: '18849496460467696128',
      // MessageGroupId: '1',
      // MessageDeduplicationId: '1',
    },
    // messageAttributes: {},
    // md5OfBody: 'e4e68fb7bd0e697a0ae8f1bb342846b3',
    awsRegion: m.region || /* istanbul ignore next */'us-west-2'
    // eventSourceARN: 'arn:aws:sqs:us-west-2:123456789012:my-queue',
  }))
});
exports.toSqsRecords = toSqsRecords;
const toSqsEventRecords = events => toSqsRecords(events.map(e => ({
  body: JSON.stringify(e, (0, _utils.compress)()),
  timestamp: e.timestamp,
  region: e.tags?.region
})));
exports.toSqsEventRecords = toSqsEventRecords;
const UNKNOWN_SQS_EVENT_TYPE = exports.UNKNOWN_SQS_EVENT_TYPE = toSqsEventRecords([{
  type: 'unknown-type'
}]);