"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.startUow = exports.functionMetrics = exports.endPipeline = exports.clear = exports.adornSqsMetrics = exports.adornKinesisMetrics = exports.adornDynamoMetrics = void 0;
var _highland = _interopRequireDefault(require("highland"));
var _bluebird = _interopRequireDefault(require("bluebird"));
var _timer = _interopRequireDefault(require("./timer"));
let funcMetrics = {};
const functionMetrics = m => {
  /* istanbul ignore next */
  if (m) funcMetrics = m;
  return funcMetrics;
};
exports.functionMetrics = functionMetrics;
const clear = opt => {
  funcMetrics = {};
  if (opt.xrayEnabled) {
    require('./xray').clearPipelineSegments();
  }
};
exports.clear = clear;
const startUow = (publishTime, batchSize) => {
  funcMetrics['stream.batch.utilization'] = batchSize / Number(process.env.BATCH_SIZE || /* istanbul ignore next */100);
  return new PipelineMetrics({
    publishTime
  });
};
exports.startUow = startUow;
const adornKinesisMetrics = (uow, event) => {
  const time = `${uow.record.kinesis.approximateArrivalTimestamp}`.padEnd(14, '0');
  uow.metrics = startUow(Number(`${time.substring(0, 10)}${time.substring(11)}`), event.Records.length);
};

/* istanbul ignore next */
exports.adornKinesisMetrics = adornKinesisMetrics;
const adornDynamoMetrics = (uow, event) => {
  uow.metrics = startUow(uow.record.dynamodb.ApproximateCreationDateTime * 1000, event.Records.length);
};

/* istanbul ignore next */
exports.adornDynamoMetrics = adornDynamoMetrics;
const adornSqsMetrics = (uow, event) => {
  uow.metrics = startUow(uow.record.attributes.SentTimestamp, event.Records.length);
};
exports.adornSqsMetrics = adornSqsMetrics;
class PipelineMetrics {
  constructor({
    pipeline,
    publishTime,
    timer,
    gauges,
    opt
  }) {
    this.pipeline = pipeline || 'default';
    this.timer = new _timer.default({
      start: publishTime || timer?.start,
      last: timer?.last,
      checkpoints: timer?.checkpoints
    });
    this.gauges = gauges || {};
    this.opt = opt;
  }
  gauge(key, value) {
    // console.log('gauge: ', key, value);
    const k = `${this.pipeline}|${key}`;
    this.gauges[k] = [...(this.gauges[k] || []), ...(Array.isArray(value) ? value : [value])];
    return this;
  }
  startPipeline({
    pipeline
  }, opt) {
    const clone = new PipelineMetrics({
      pipeline,
      timer: this.timer,
      opt
    });

    // time waiting on channel capacity (e.g. shard count)
    clone.timer.checkpoint(`${pipeline}|stream.channel.wait.time`);

    // Initialize an xray segment if enabled
    if (opt.xrayEnabled) {
      clone.xraySegment = require('./xray').startPipelineSegment(pipeline);
    }
    return clone;
  }
  endPipeline() {
    this.timer.end(`${this.pipeline}|stream.pipeline.time`);
    return this;
  }
  startStep(step) {
    /* istanbul ignore else */
    if (this.opt?.metrics.enabled('step')) {
      // time waiting for io capacity (e.g parallel count)
      this.timer.checkpoint(`${this.pipeline}|${step}|stream.pipeline.io.wait.time`);
    }
    return this;
  }
  endStep(step) {
    /* istanbul ignore else */
    if (this.opt?.metrics.enabled('step')) {
      this.timer.checkpoint(`${this.pipeline}|${step}|stream.pipeline.io.time`);
    }
    return this;
  }

  // wrap promise
  w(p, step) {
    const self = this;
    self.startStep(step);
    return _bluebird.default.resolve(p()) // now start the promise
    .tap(() => self.endStep(step));
  }
}
const endPipeline = (pipelineId, opt, s) => s.consume((err, x, push, next) => {
  /* istanbul ignore if */
  if (err) {
    push(err);
    next();
  } else if (x === _highland.default.nil) {
    if (opt.xrayEnabled) {
      require('./xray').terminateSegment(pipelineId);
    }
    push(null, x);
  } else {
    // per uow
    x.metrics?.endPipeline();
    push(null, x);
    next();
  }
});
exports.endPipeline = endPipeline;