"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.initializeFrom = exports.initialize = void 0;
var _highland = _interopRequireDefault(require("highland"));
var _memoryCache = _interopRequireDefault(require("memory-cache"));
var _faults = require("../faults");
var _utils = require("../utils");
const debug = (0, _utils.debug)('pl:init');
let thePipelines = {};
const initialize = (pipelines, opt = {}) => {
  const keys = Object.keys(pipelines);
  (0, _utils.options)(opt);
  debug('initialize: %j', keys);
  thePipelines = keys.reduce((accumulator, id) => ({
    ...accumulator,
    [id]: pipelines[id]({
      // pass in options
      id,
      ...opt,
      ...addDebug(id),
      ...addEncryptors(opt)
    })
  }), {});
  _memoryCache.default.clear();
  return {
    assemble: assemble(opt)
  };
};
exports.initialize = initialize;
const initializeFrom = rules => rules.reduce((accumulator, rule) => ({
  ...accumulator,
  [rule.id]: opt => (rule.pattern || rule.flavor)({
    ...rule,
    // included 1st so rules are printed 1st in debug output
    ...opt,
    ...rule,
    // include again for override precedence
    ...addEncryptors({
      ...opt,
      ...rule
    })
  })
}), {});
exports.initializeFrom = initializeFrom;
const assemble = opt => (head, includeFaultHandler = true) => {
  const keys = Object.keys(thePipelines).filter(k => !process.env.DISABLED_PIPELINES?.includes(k));
  debug('assemble: %j', keys);
  if (includeFaultHandler) {
    // after pre processoring
    head = head.errors(_faults.faults);
  }
  const lines = keys.map(key => {
    const f = thePipelines[key];
    const p = _highland.default.pipeline(f);
    p.id = key;
    return p;
  });

  /* istanbul ignore else */
  if (lines.length > 0) {
    const last = lines.length - 1;
    lines.slice(0, last).forEach((p, i) => {
      debug('FORK: %s', p.id);
      const os = head.observe();
      lines[i] = os.map(initPipeline(p.id)).tap(startPipeline(opt)).through(p).through(endPipeline(opt, p.id));
    });
    debug('FORK: %s', lines[last].id);
    const p = lines[last];
    lines[last] = head.map(initPipeline(p.id)).tap(startPipeline(opt)).through(lines[last]).through(endPipeline(opt, p.id));
  }
  let s = (0, _highland.default)(lines).merge();
  if (includeFaultHandler) {
    s = s.errors(_faults.faults).through((0, _faults.flushFaults)({
      ...opt,
      ...addDebug('fault')
    }));
  }
  return s;
};
const addDebug = id => ({
  debug: (0, _utils.debug)(`pl:${id}`)
});
const initPipeline = pipeline => uow => ({
  // shallow clone of data per pipeline
  pipeline,
  ...uow,
  ...addDebug(pipeline)
});
const startPipeline = opt => uow => {
  if (opt.metrics) {
    uow.metrics = uow.metrics.startPipeline(uow, opt);
  }
};
const endPipeline = (opt, pipelineId) => s => opt.metrics ? opt.metrics.endPipeline(pipelineId, opt, s) : s;
const addEncryptors = opt => ({
  encrypt: (0, _utils.encryptData)(opt),
  decrypt: (0, _utils.decryptData)(opt)
});