"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.updateExpression = exports.updateDynamoDB = exports.timestampCondition = exports.putDynamoDB = exports.pkCondition = void 0;
var _highland = _interopRequireDefault(require("highland"));
var _merge = _interopRequireDefault(require("lodash/merge"));
var _dynamodb = _interopRequireDefault(require("../connectors/dynamodb"));
var _faults = require("../utils/faults");
var _print = require("../utils/print");
var _ratelimit = require("../utils/ratelimit");
const updateExpression = Item => {
  const keys = Object.keys(Item);
  const ExpressionAttributeNames = keys.filter(attrName => Item[attrName] !== undefined).map(attrName => ({
    [`#${attrName}`]: attrName
  })).reduce(_merge.default, {});
  const ExpressionAttributeValues = keys.filter(attrName => Item[attrName] !== undefined && Item[attrName] !== null).map(attrName => ({
    [`:${attrName}`]: Item[attrName]
  })).reduce(_merge.default, {});
  let UpdateExpression = `SET ${keys.filter(attrName => Item[attrName] !== undefined && Item[attrName] !== null).map(attrName => `#${attrName} = :${attrName}`).join(', ')}`;
  const UpdateExpressionRemove = keys.filter(attrName => Item[attrName] === null).map(attrName => `#${attrName}`).join(', ');
  if (UpdateExpressionRemove.length) {
    UpdateExpression = `${UpdateExpression} REMOVE ${UpdateExpressionRemove}`;
  }
  return {
    ExpressionAttributeNames,
    ExpressionAttributeValues,
    UpdateExpression,
    ReturnValues: 'ALL_NEW'
  };
};
exports.updateExpression = updateExpression;
const timestampCondition = (fieldName = 'timestamp') => ({
  ConditionExpression: `attribute_not_exists(#${fieldName}) OR #${fieldName} < :${fieldName}`
});
exports.timestampCondition = timestampCondition;
const pkCondition = (fieldName = 'pk') => ({
  ConditionExpression: `attribute_not_exists(${fieldName})`
});
exports.pkCondition = pkCondition;
const updateDynamoDB = ({
  id: pipelineId,
  debug = (0, _print.debug)('dynamodb'),
  tableName = process.env.ENTITY_TABLE_NAME || process.env.EVENT_TABLE_NAME,
  updateRequestField = 'updateRequest',
  updateResponseField = 'updateResponse',
  parallel = Number(process.env.UPDATE_PARALLEL) || Number(process.env.PARALLEL) || 4,
  timeout = Number(process.env.DYNAMODB_TIMEOUT) || Number(process.env.TIMEOUT) || 1000,
  removeUndefinedValues = true,
  step = 'save',
  ...opt
} = {}) => {
  const connector = new _dynamodb.default({
    pipelineId,
    debug,
    tableName,
    timeout,
    removeUndefinedValues,
    ...opt
  });
  const invoke = uow => {
    if (!uow[updateRequestField]) return (0, _highland.default)(Promise.resolve(uow));
    const p = () => connector.update(uow[updateRequestField], uow).then(updateResponse => ({
      ...uow,
      [updateResponseField]: updateResponse
    })).catch((0, _faults.rejectWithFault)(uow));
    return (0, _highland.default)(uow.metrics?.w(p, step) || p()); // wrap promise in a stream
  };
  return s => s.through((0, _ratelimit.ratelimit)(opt)).map(invoke).parallel(parallel);
};
exports.updateDynamoDB = updateDynamoDB;
const putDynamoDB = ({
  id: pipelineId,
  debug = (0, _print.debug)('dynamodb'),
  tableName = process.env.EVENT_TABLE_NAME || process.env.ENTITY_TABLE_NAME,
  putRequestField = 'putRequest',
  parallel = Number(process.env.UPDATE_PARALLEL) || Number(process.env.PARALLEL) || 4,
  timeout = Number(process.env.DYNAMODB_TIMEOUT) || Number(process.env.TIMEOUT) || 1000,
  step = 'save',
  ...opt
} = {}) => {
  const connector = new _dynamodb.default({
    pipelineId,
    debug,
    tableName,
    timeout,
    ...opt
  });
  const invoke = uow => {
    if (!uow[putRequestField]) return (0, _highland.default)(Promise.resolve(uow));
    const p = () => connector.put(uow[putRequestField], uow).then(putResponse => ({
      ...uow,
      putResponse
    })).catch((0, _faults.rejectWithFault)(uow));
    return (0, _highland.default)(uow.metrics?.w(p, step) || p()); // wrap promise in a stream
  };
  return s => s.map(invoke).parallel(parallel);
};
exports.putDynamoDB = putDynamoDB;