"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.publishToEventBridge = void 0;
var _highland = _interopRequireDefault(require("highland"));
var _eventbridge = _interopRequireDefault(require("../connectors/eventbridge"));
var _batch = require("../utils/batch");
var _faults = require("../utils/faults");
var _print = require("../utils/print");
var _tags = require("../utils/tags");
var _compression = require("../utils/compression");
var _ratelimit = require("../utils/ratelimit");
const publishToEventBridge = ({
  // eslint-disable-line import/prefer-default-export
  id: pipelineId,
  debug = (0, _print.debug)('eventbridge'),
  busName = process.env.BUS_NAME || 'undefined',
  source = process.env.BUS_SRC || 'custom',
  // could change this to internal vs external/ingress/egress
  eventField = 'event',
  // is often named emit
  publishRequestEntryField = 'publishRequestEntry',
  publishRequestField = 'publishRequest',
  // was inputParams
  maxPublishRequestSize = Number(process.env.PUBLISH_MAX_REQ_SIZE) || Number(process.env.MAX_REQ_SIZE) || 256 * 1024,
  batchSize = Number(process.env.PUBLISH_BATCH_SIZE) || Number(process.env.BATCH_SIZE) || 10,
  parallel = Number(process.env.PUBLISH_PARALLEL) || Number(process.env.PARALLEL) || 8,
  handleErrors = true,
  retryConfig,
  step = 'publish',
  ...opt
} = {}) => {
  const connector = new _eventbridge.default({
    pipelineId,
    debug,
    retryConfig,
    ...opt
  });
  const toPublishRequestEntry = uow => ({
    ...uow,
    [publishRequestEntryField]: uow[eventField] ? {
      EventBusName: busName,
      Source: source,
      DetailType: uow[eventField].type,
      Detail: JSON.stringify(uow[eventField], (0, _compression.compress)(uow[eventField].type !== 'fault' ? opt : {
        ...opt,
        compressionIgnore: _faults.FAULT_COMPRESSION_IGNORE
      }))
    } : undefined
  });
  const toPublishRequest = batchUow => ({
    ...batchUow,
    [publishRequestField]: {
      Entries: batchUow.batch.filter(uow => uow[publishRequestEntryField]).map(uow => uow[publishRequestEntryField])
    }
  });
  const putEvents = batchUow => {
    if (!batchUow[publishRequestField].Entries.length) {
      return (0, _highland.default)(Promise.resolve(batchUow));
    }
    const p = () => connector.putEvents(batchUow[publishRequestField], batchUow).catch((0, _faults.rejectWithFault)(batchUow, !handleErrors)).then(publishResponse => ({
      ...batchUow,
      publishResponse
    }));
    return (0, _highland.default)(batchUow.batch[0].metrics?.w(p, step) || p()); // wrap promise in a stream
  };
  return s => s.map((0, _tags.adornStandardTags)(eventField)).through((0, _ratelimit.ratelimit)(opt)).map(toPublishRequestEntry).consume((0, _batch.batchWithSize)({
    ...opt,
    batchSize,
    maxRequestSize: maxPublishRequestSize,
    requestEntryField: publishRequestEntryField,
    requestField: publishRequestField,
    debug
  })).map(_batch.toBatchUow).map(toPublishRequest).map(putEvents).parallel(parallel).flatMap(_batch.unBatchUow); // for cleaner logging and testing
};
exports.publishToEventBridge = publishToEventBridge;