"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.invokeLambda = void 0;
var _highland = _interopRequireDefault(require("highland"));
var _lambda = _interopRequireDefault(require("../connectors/lambda"));
var _faults = require("../utils/faults");
var _print = require("../utils/print");
var _ratelimit = require("../utils/ratelimit");
const invokeLambda = ({
  // eslint-disable-line import/prefer-default-export
  id: pipelineId,
  debug = (0, _print.debug)('lambda'),
  invokeField = 'invokeRequest',
  parallel = Number(process.env.LAMBDA_PARALLEL) || Number(process.env.PARALLEL) || 8,
  step = 'invoke',
  ...opt
} = {}) => {
  const connector = new _lambda.default({
    pipelineId,
    debug,
    ...opt
  });
  const invoke = uow => {
    const p = () => connector.invoke(uow[invokeField], uow).then(invokeResponse => ({
      ...uow,
      invokeResponse
    })).catch((0, _faults.rejectWithFault)(uow));
    return (0, _highland.default)(uow.metrics?.w(p, step) || p()); // wrap promise in a stream
  };
  return s => s.through((0, _ratelimit.ratelimit)(opt)).map(invoke).parallel(parallel);
};
exports.invokeLambda = invokeLambda;