"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.putObjectToS3 = exports.deleteObjectFromS3 = exports.copyS3Object = void 0;
var _highland = _interopRequireDefault(require("highland"));
var _s = _interopRequireDefault(require("../connectors/s3"));
var _faults = require("../utils/faults");
var _print = require("../utils/print");
var _ratelimit = require("../utils/ratelimit");
const putObjectToS3 = ({
  debug = (0, _print.debug)('s3'),
  id: pipelineId,
  bucketName = process.env.BUCKET_NAME,
  putRequestField = 'putRequest',
  putResponseField = 'putResponse',
  parallel = Number(process.env.S3_PARALLEL) || Number(process.env.PARALLEL) || 8,
  step = 'save',
  ...opt
} = {}) => {
  const connector = new _s.default({
    pipelineId,
    debug,
    bucketName,
    ...opt
  });
  const putObject = uow => {
    if (!uow[putRequestField]) return (0, _highland.default)(Promise.resolve(uow));
    const p = () => connector.putObject(uow[putRequestField], uow).then(putResponse => ({
      ...uow,
      [putResponseField]: putResponse
    })).catch((0, _faults.rejectWithFault)(uow));
    return (0, _highland.default)(uow.metrics?.w(p, step) || p()); // wrap promise in a stream
  };
  return s => s.through((0, _ratelimit.ratelimit)(opt)).map(putObject).parallel(parallel);
};
exports.putObjectToS3 = putObjectToS3;
const deleteObjectFromS3 = ({
  debug,
  id: pipelineId,
  bucketName = process.env.BUCKET_NAME,
  deleteRequestField = 'deleteRequest',
  deleteResponseField = 'deleteResponse',
  parallel = Number(process.env.S3_PARALLEL) || Number(process.env.PARALLEL) || 8,
  step = 'delete',
  ...opt
} = {}) => {
  const connector = new _s.default({
    pipelineId,
    debug,
    bucketName,
    ...opt
  });
  const deleteObject = uow => {
    if (!uow[deleteRequestField]) return (0, _highland.default)(Promise.resolve(uow));
    const p = () => connector.deleteObject(uow[deleteRequestField]).then(deleteResponse => ({
      ...uow,
      [deleteResponseField]: deleteResponse
    })).catch((0, _faults.rejectWithFault)(uow));
    return (0, _highland.default)(uow.metrics?.w(p, step) || p()); // wrap promise in a stream
  };
  return s => s.map(deleteObject).parallel(parallel);
};
exports.deleteObjectFromS3 = deleteObjectFromS3;
const copyS3Object = ({
  debug,
  id: pipelineId,
  bucketName = process.env.BUCKET_NAME,
  copyRequestField = 'copyRequest',
  copyResponseField = 'copyResponse',
  parallel = Number(process.env.S3_PARALLEL) || Number(process.env.PARALLEL) || 8,
  step = 'copy',
  ...opt
} = {}) => {
  const connector = new _s.default({
    pipelineId,
    debug,
    bucketName,
    ...opt
  });
  const copyObject = uow => {
    if (!uow[copyRequestField]) return (0, _highland.default)(Promise.resolve(uow));
    const p = () => connector.copyObject(uow[copyRequestField]).then(copyResponse => ({
      ...uow,
      [copyResponseField]: copyResponse
    })).catch((0, _faults.rejectWithFault)(uow));
    return (0, _highland.default)(uow.metrics?.w(p, step) || p());
  };
  return s => s.map(copyObject).parallel(parallel);
};
exports.copyS3Object = copyS3Object;