"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.publishToSns = void 0;
var _highland = _interopRequireDefault(require("highland"));
var _sns = _interopRequireDefault(require("../connectors/sns"));
var _faults = require("../utils/faults");
var _print = require("../utils/print");
var _ratelimit = require("../utils/ratelimit");
const publishToSns = ({
  // eslint-disable-line import/prefer-default-export
  id: pipelineId,
  debug = (0, _print.debug)('sns'),
  topicArn = process.env.TOPIC_ARN,
  messageField = 'message',
  parallel = Number(process.env.SNS_PARALLEL) || Number(process.env.PARALLEL) || 8,
  step = 'publish',
  ...opt
} = {}) => {
  const connector = new _sns.default({
    pipelineId,
    debug,
    topicArn,
    ...opt
  });
  const publish = uow => {
    const p = () => connector.publish(uow[messageField], uow).then(publishResponse => ({
      ...uow,
      publishResponse
    })).catch((0, _faults.rejectWithFault)(uow));
    return (0, _highland.default)(uow.metrics?.w(p, step) || /* istanbul ignore next */p()); // wrap promise in a stream
  };
  return s => s.through((0, _ratelimit.ratelimit)(opt)).map(publish).parallel(parallel);
};
exports.publishToSns = publishToSns;