"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.throwFault = exports.rejectWithFault = exports.faultyAsyncStream = exports.faultyAsync = exports.faulty = exports.faultify = exports.FAULT_COMPRESSION_IGNORE = void 0;
var _highland = _interopRequireDefault(require("highland"));
var _bluebird = _interopRequireDefault(require("bluebird"));
// the kinesis base64 data field isn't compressable
const FAULT_COMPRESSION_IGNORE = exports.FAULT_COMPRESSION_IGNORE = ['uow', 'batch', 'record', 'kinesis', 'data'];
const throwFault = (uow, ignore = false) => err => {
  /* istanbul ignore else */
  if (!ignore) {
    // adorn the troubled uow
    // for processing in the errors handler
    err.uow = uow;
  }
  throw err;
};
exports.throwFault = throwFault;
const rejectWithFault = (uow, ignore = false) => err => {
  /* istanbul ignore else */
  if (!ignore) {
    // adorn the troubled uow
    // for processing in the errors handler
    err.uow = uow;
  }
  return _bluebird.default.reject(err);
};
exports.rejectWithFault = rejectWithFault;
const faulty = (funct, ignore = false) => (uow, ...args) => {
  // eslint-disable-line consistent-return
  try {
    return funct(uow, ...args);
  } catch (e) {
    throwFault(uow, ignore)(e);
  }
};
exports.faulty = faulty;
const faultyAsync = (funct, ignore = false) => (uow, ...args) => funct(uow, ...args).catch(rejectWithFault(uow, ignore));
exports.faultyAsync = faultyAsync;
const faultyAsyncStream = (funct, ignore = false) => (...args) => (0, _highland.default)(faultyAsync(funct, ignore)(...args));
exports.faultyAsyncStream = faultyAsyncStream;
const faultify = func => (...args) => new _bluebird.default((resolve, reject) => {
  try {
    resolve(func(...args));
  } catch (e) {
    reject(e);
  }
});
exports.faultify = faultify;